from .length import Length
from .metric import *
from .imperial import *

__all__ = ['Length']

Length.Millimeter = Millimeter
Length.Centimeter = Centimeter
Length.Decimeter = Decimeter
Length.Meter = Meter
Length.Decameter = Decameter
Length.Hectometer = Hectometer
Length.Kilometer = Kilometer

Length.Line = Line
Length.Inch = Inch
Length.Foot = Foot
Length.Yard = Yard
Length.Mile = Mile
